#include "stdafx.hpp"

#include "ActionListener.hpp"

ActionListener::ActionListener( const ActionSettingInfo& v_actionSettingInfo )
	: maxProcess_( v_actionSettingInfo.getMaxProcess() )
	, processCount_( 0 )
{
	ActionInvokerFactory actionInvokerFactory;
	pActionInvoker_ = actionInvokerFactory.create( v_actionSettingInfo );
}

ActionListener::~ActionListener() throw()
{
}

void ActionListener::sweepEvent() throw()
{
	if( pActionInvoker_.get() ) {
		processCount_ = pActionInvoker_->sweepTerminatedProcess();
	}
	else {
		processCount_ = 0;
	}
}

bool ActionListener::isEventListenerAvailable() const throw()
{
	bool result = false;
	if( pActionInvoker_.get() ) {
		result = ( maxProcess_ == 0 ) || ( maxProcess_ > processCount_ );
	}
	return result;
}

void ActionListener::notifyFileEvent( DWORD v_eventId, const tstring& v_absolutePath, const FileInfo& v_fileInfo ) throw()
{
	if( v_eventId == 0 ) { //TODO:MSGID
		if( pActionInvoker_.get() ) {
			if( pActionInvoker_->createProcess( v_absolutePath, v_fileInfo ) ) {
				processCount_ = pActionInvoker_->sweepTerminatedProcess();
			}
		}
	}
}

tstring ActionListener::getDescription() const throw()
{
	if( pActionInvoker_.get() ) {
		const ActionSettingInfo& settingInfo = pActionInvoker_->getSettingInfo();
		const tstring actionTypeName = pActionInvoker_->getDescription();
		
		tstring description;
		if( ! settingInfo.isAvailable() ) {
			description += _TEXT( "<<DISABLE>> " );
		}
		else {
			description += _TEXT( "<" ) + pActionInvoker_->getDescription() + _TEXT( "> " );
			description += settingInfo.getAppName() + _TEXT(" ") + settingInfo.getParam();
		}

		return description;
	}
	return tstring( _TEXT( "<<NOTHING>>" ) );
}

void ActionListener::notifyEvent( DWORD v_eventId ) throw()
{
	// no action.
}

