#include "stdafx.hpp"

#include "DirWatchObject.hpp"
#include "tstringUty.hpp"

DirWatchObject::DirWatchObject(
	const tstring& v_watchDir,
	bool v_enableSubDir,
	const CFileTimeSpan& v_retrySpan,
	const CFileTimeSpan& v_expirySpan,
	bool v_enableAPI
	)
	: watchDir_( tstringuty::ensureEndsWith( v_watchDir, _TEXT("\\") ) )
	, enableSubDir_( v_enableSubDir )
	, enableAPI_( v_enableAPI )
	, hWatchHandle_( NULL )
	, expiryTimer_( v_expirySpan, 0, false )
	, retryTimer_( v_retrySpan, 0, false )
	, bNewHandle_( false )
{
}

DirWatchObject::~DirWatchObject()
{
	if( hWatchHandle_ != NULL ) {
		::FindCloseChangeNotification( hWatchHandle_ );
	}
}

HANDLE DirWatchObject::getHandle()
{
	const CFileTime now = CFileTime::GetCurrentTime();
	if( enableAPI_ && ( hWatchHandle_ == NULL || expiryTimer_.isSignaled( now ) ) ) {
		if( ! retryTimer_.isEnabled() || retryTimer_.isSignaled( now ) ) {
			::OutputDebugString( _TEXT("Ďnh̍쐬s܂B\n") );
			HANDLE watchHandle = ::FindFirstChangeNotification(
				watchDir_.c_str(),
				enableSubDir_,
				FILE_NOTIFY_CHANGE_FILE_NAME |
				FILE_NOTIFY_CHANGE_DIR_NAME |
				FILE_NOTIFY_CHANGE_LAST_WRITE |
				FILE_NOTIFY_CHANGE_SIZE |
				FILE_NOTIFY_CHANGE_ATTRIBUTES
				);
			if( watchHandle != INVALID_HANDLE_VALUE ) {
				::OutputDebugString( _TEXT("Ďnh̍쐬ɐ܂B\n") );
				if( hWatchHandle_ != NULL ) {
					// ÂnhjB
					::FindCloseChangeNotification( hWatchHandle_ );
				}
				hWatchHandle_ = watchHandle;
				expiryTimer_.reset( now, expiryTimer_.getSpan() > 0 );
				retryTimer_.reset( now, false );
				bNewHandle_ = true;
			}
			else {
				::OutputDebugString( _TEXT("Ďnh̍쐬Ɏs܂B\n") );
				retryTimer_.reset( now, true );
				bNewHandle_ = false;
			}
		}
	}
	return hWatchHandle_;
}

bool DirWatchObject::isNewHandle() const
{
	return bNewHandle_;
}

void DirWatchObject::next()
{
	if( hWatchHandle_ != NULL ) {
		bNewHandle_ = false;
		if( ! ::FindNextChangeNotification( hWatchHandle_ ) ) {
			// ̑ҋ@Ɏs = nh̉\邽߁Ač쐬ׂłB
			::FindCloseChangeNotification( hWatchHandle_ );
			hWatchHandle_ = NULL;
		}
		else {
			const CFileTime now = CFileTime::GetCurrentTime();
			expiryTimer_.reset( now, expiryTimer_.getSpan() > 0 ); // ɓ삵Ă̂ŔjԂB
		}
	}
}


